@@ -6,7 +6,7 @@
 #include <stdlib.h>
 #include <stdarg.h>
 
-#if defined(HAVE_PTHREAD_JIT)
+#ifdef __APPLE__
   #include <pthread.h>
 #endif
 
@@ -22,6 +22,7 @@
 #include <orc/orcprogram.h>
 #include <orc/orcdebug.h>
 #include <orc/orcinternal.h>
+#include <orc/orcutils-private.h>
 
 #ifdef HAVE_VALGRIND_VALGRIND_H
 #include <valgrind/valgrind.h>
@@ -64,6 +65,17 @@ int _orc_compiler_flag_randomize;
 /* For Windows */
 int _orc_codemem_alignment;
 
+#if defined(MAC_OS_VERSION_11_0) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_VERSION_11_0
+G_ALWAYS_INLINE
+static void
+orc_pthread_jit_write_protect_np (int protect)
+{
+  if (__builtin_available (macOS 11.0, *))
+    if (pthread_jit_write_protect_supported_np ())
+      pthread_jit_write_protect_np (protect);
+}
+#endif
+
 void
 _orc_compiler_init (void)
 {
@@ -126,11 +138,6 @@ _orc_compiler_init (void)
     }
   }
 #endif
-
-#if defined(HAVE_PTHREAD_JIT)
-  ORC_INFO("pthread_jit_write_protect_supported_np() = %i",
-      pthread_jit_write_protect_supported_np());
-#endif
 }
 
 int
@@ -456,8 +463,8 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
   program->orccode->code_size = compiler->codeptr - compiler->code;
   orc_code_allocate_codemem (program->orccode, program->orccode->code_size);
 
-#if defined(HAVE_PTHREAD_JIT)
-  pthread_jit_write_protect_np(0);
+#if defined(MAC_OS_VERSION_11_0) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_VERSION_11_0
+  orc_pthread_jit_write_protect_np (0);
 #endif
 #if defined(HAVE_CODEMEM_VIRTUALALLOC)
   /* Ensure that code region is writable before memcpy */
@@ -475,8 +482,8 @@ orc_program_compile_full (OrcProgram *program, OrcTarget *target,
     compiler->target->flush_cache (program->orccode);
   }
 
-#if defined(HAVE_PTHREAD_JIT)
-  pthread_jit_write_protect_np(1);
+#if defined(MAC_OS_VERSION_11_0) && MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_VERSION_11_0
+  orc_pthread_jit_write_protect_np (1);
 #endif
 #if defined(HAVE_CODEMEM_VIRTUALALLOC)
   /* Code region is now ready for execution */
