@@ -53,38 +53,39 @@ typedef enum {
   MODE_IMPL,
   MODE_HEADER,
   MODE_TEST,
-  MODE_ASSEMBLY
+  MODE_ASSEMBLY,
+  MODE_PARSE
 } OrcMode;
 
 OrcMode mode = MODE_IMPL;
 
 void help (void)
 {
-  printf("Usage:\n");
-  printf("  orcc [OPTION...] INPUT_FILE\n");
-  printf("\n");
-  printf("Help Options:\n");
-  printf("  -h, --help              Show help options\n");
-  printf("\n");
-  printf("Application Options:\n");
-  printf("  -v, --verbose           Output more information\n");
-  printf("  -o, --output FILE       Write output to FILE\n");
-  printf("  --implementation        Produce C code implementing functions\n");
-  printf("  --header                Produce C header for functions\n");
-  printf("  --test                  Produce test code for functions\n");
-  printf("  --assembly              Produce assembly code for functions\n");
-  printf("  --include FILE          Add #include <FILE> to code\n");
-  printf("  --target TARGET         Generate assembly for TARGET\n");
-  printf("  --compat VERSION        Generate code compatible with Orc version VERSION\n");
-  printf("  --inline                Generate inline functions in header\n");
-  printf("  --no-inline             Do not generate inline functions in header\n");
-  printf("  --internal              Mark functions in header for internal visibility\n");
-  printf("  --no-internal           Do not mark functions in header for internal visibility\n");
-  printf("  --decorator DECORATOR   Decorate functions in header with DECORATOR\n");
-  printf("  --init-function FUNCTION  Generate initialization function\n");
-  printf("  --lazy-init             Do Orc compile at function execution\n");
-  printf("  --no-backup             Do not generate backup functions\n");
-  printf("\n");
+  fprintf(stderr, "Usage:\n");
+  fprintf(stderr, "  orcc [OPTION...] INPUT_FILE\n");
+  fprintf(stderr, "\n");
+  fprintf(stderr, "Help Options:\n");
+  fprintf(stderr, "  -h, --help              Show help options\n");
+  fprintf(stderr, "\n");
+  fprintf(stderr, "Application Options:\n");
+  fprintf(stderr, "  -v, --verbose           Output more information\n");
+  fprintf(stderr, "  -o, --output FILE       Write output to FILE\n");
+  fprintf(stderr, "  --implementation        Produce C code implementing functions\n");
+  fprintf(stderr, "  --header                Produce C header for functions\n");
+  fprintf(stderr, "  --test                  Produce test code for functions\n");
+  fprintf(stderr, "  --assembly              Produce assembly code for functions\n");
+  fprintf(stderr, "  --include FILE          Add #include <FILE> to code\n");
+  fprintf(stderr, "  --target TARGET         Generate assembly for TARGET\n");
+  fprintf(stderr, "  --compat VERSION        Generate code compatible with Orc version VERSION\n");
+  fprintf(stderr, "  --inline                Generate inline functions in header\n");
+  fprintf(stderr, "  --no-inline             Do not generate inline functions in header\n");
+  fprintf(stderr, "  --internal              Mark functions in header for internal visibility\n");
+  fprintf(stderr, "  --no-internal           Do not mark functions in header for internal visibility\n");
+  fprintf(stderr, "  --decorator DECORATOR   Decorate functions in header with DECORATOR\n");
+  fprintf(stderr, "  --init-function FUNCTION  Generate initialization function\n");
+  fprintf(stderr, "  --lazy-init             Do Orc compile at function execution\n");
+  fprintf(stderr, "  --no-backup             Do not generate backup functions\n");
+  fprintf(stderr, "\n");
 
   exit (0);
 }
@@ -94,14 +95,14 @@ main (int argc, char *argv[])
 {
   const char *orc_version;
   char *code;
-  int n;
   int i;
   char *output_file = NULL;
   char *input_file = NULL;
   char *include_file = NULL;
   char *compat_version = VERSION;
   FILE *output;
-  char *log = NULL;
+  OrcParseError **errors = NULL;
+  int n_errors = 0;
 
   orc_init ();
 
@@ -124,6 +125,8 @@ main (int argc, char *argv[])
       mode = MODE_TEST;
     } else if (strcmp(argv[i], "--assembly") == 0) {
       mode = MODE_ASSEMBLY;
+    } else if (strcmp(argv[i], "--parse-only") == 0) {
+      mode = MODE_PARSE;
     } else if (strcmp(argv[i], "--include") == 0) {
       if (i+1 < argc) {
         include_file = argv[i+1];
@@ -176,7 +179,7 @@ main (int argc, char *argv[])
         strcmp(argv[i], "-v") == 0) {
       verbose = 1;
     } else if (strcmp(argv[i], "--version") == 0) {
-      printf("Orc Compiler " PACKAGE_VERSION "\n");
+      fprintf(stderr, "Orc Compiler " PACKAGE_VERSION "\n");
       exit (0);
     } else if (strcmp(argv[i], "--compat") == 0) {
       if (i+1 < argc) {
@@ -190,25 +193,25 @@ main (int argc, char *argv[])
     } else if (strcmp(argv[i], "--no-backup") == 0) {
       use_backup = FALSE;
     } else if (strncmp(argv[i], "-", 1) == 0) {
-      printf("Unknown option: %s\n", argv[i]);
+      fprintf(stderr, "Unknown option: %s\n", argv[i]);
       exit (1);
     } else {
       if (input_file == NULL) {
         input_file = argv[i];
       } else {
-        printf("More than one input file specified: %s\n", argv[i]);
+        fprintf(stderr, "More than one input file specified: %s\n", argv[i]);
         exit (1);
       }
     }
   }
 
   if (input_file == NULL) {
-    printf("No input file specified\n");
+    fprintf(stderr, "No input file specified\n");
     exit (1);
   }
 
   if (mode == MODE_ASSEMBLY && orc_target_get_by_name (target) == NULL) {
-    printf("Unknown target \"%s\"\n", target);
+    fprintf(stderr, "Unknown target \"%s\"\n", target);
     exit (1);
   }
 
@@ -219,13 +222,13 @@ main (int argc, char *argv[])
     n = sscanf (compat_version, "%d.%d.%d.%d", &major, &minor, &micro, &nano);
 
     if (n < 3) {
-      printf("Unknown version \"%s\"\n", compat_version);
+      fprintf(stderr, "Unknown version \"%s\"\n", compat_version);
       exit (1);
     }
 
     compat = ORC_VERSION(major,minor,micro,nano);
     if (compat < ORC_VERSION(0,4,5,0)) {
-      printf("Compatibility version \"%s\" not supported.  Minimum 0.4.5\n",
+      fprintf(stderr, "Compatibility version \"%s\" not supported.  Minimum 0.4.5\n",
           compat_version);
       exit (1);
     }
@@ -248,26 +251,45 @@ main (int argc, char *argv[])
       case MODE_ASSEMBLY:
         output_file = "out.s";
         break;
+      case MODE_PARSE:
+        output_file = NULL;
+        break;
     }
   }
 
   code = read_file (input_file);
   if (!code) {
-    printf("Could not read input file: %s\n", input_file);
+    fprintf(stderr, "Could not read input file: %s\n", input_file);
     exit(1);
   }
 
-  n = orc_parse_full (code, &programs, &log);
+  orc_parse_code (code, &programs, &n_programs, &errors, &n_errors);
   free(code);
-  n_programs = n;
-  printf("%s", log);
-  free(log);
+  if (n_errors > 0) {
+    int i;
+    for (i=0;i<n_errors;i++) {
+      fprintf(stderr, "%s @ %i: error: %s\n", errors[i]->source, errors[i]->line_number, errors[i]->text);
+    }
+    orc_parse_error_freev (errors);
+    exit (1);
+  }
 
   if (programs == NULL) {
-    printf("no programs\n");
+    if (verbose) {
+      fprintf(stderr, "no programs found\n");
+    }
     exit(1);
   }
 
+  if (verbose) {
+    fprintf(stderr, "%i program%s parsed\n",
+           n_programs, (n_programs > 1) ?"s" :"");
+  }
+
+  if (mode == MODE_PARSE) {
+    exit (0);
+  }
+
   if (init_function == NULL) {
     init_function = orc_parse_get_init_function (programs[0]);
   }
@@ -278,7 +300,7 @@ main (int argc, char *argv[])
 
   output = fopen (output_file, "w");
   if (!output) {
-    printf("Could not write output file: %s\n", output_file);
+    fprintf(stderr, "Could not write output file: %s\n", output_file);
     exit(1);
   }
 
@@ -299,7 +321,7 @@ main (int argc, char *argv[])
     fprintf(output, "#ifndef DISABLE_ORC\n");
     fprintf(output, "#include <orc/orc.h>\n");
     fprintf(output, "#endif\n");
-    for(i=0;i<n;i++){
+    for(i=0;i<n_programs;i++){
       output_code_header (programs[i], output);
     }
     if (init_function) {
@@ -309,7 +331,7 @@ main (int argc, char *argv[])
     fprintf(output, "\n");
     fprintf(output, "%s", orc_target_get_asm_preamble ("c"));
     fprintf(output, "\n");
-    for(i=0;i<n;i++){
+    for(i=0;i<n_programs;i++){
       output_code (programs[i], output);
     }
     fprintf(output, "\n");
@@ -336,14 +358,14 @@ main (int argc, char *argv[])
     if (!use_inline) {
       fprintf(output, "\n");
       fprintf(output, "%s", orc_target_c_get_typedefs ());
-      for(i=0;i<n;i++){
+      for(i=0;i<n_programs;i++){
         output_code_header (programs[i], output);
       }
     } else {
       fprintf(output, "\n");
       fprintf(output, "#include <orc/orc.h>\n");
       fprintf(output, "\n");
-      for(i=0;i<n;i++){
+      for(i=0;i<n_programs;i++){
         output_code_execute (programs[i], output, TRUE);
       }
     }
@@ -367,7 +389,7 @@ main (int argc, char *argv[])
     fprintf(output, "%s", orc_target_get_asm_preamble ("c"));
     fprintf(output, "\n");
     if (use_backup) {
-      for(i=0;i<n;i++){
+      for(i=0;i<n_programs;i++){
         fprintf(output, "/* %s */\n", programs[i]->name);
         output_code_backup (programs[i], output);
       }
@@ -378,13 +400,13 @@ main (int argc, char *argv[])
     fprintf(output, "\n");
     fprintf(output, "static void help (const char *argv0)\n");
     fprintf(output, "{\n");
-    fprintf(output, "  printf(\"Usage:\\n\");\n");
-    fprintf(output, "  printf(\"  %%s [OPTION]\\n\", argv0);\n");
-    fprintf(output, "  printf(\"Help Options:\\n\");\n");
-    fprintf(output, "  printf(\"  -h, --help          Show help options\\n\");\n");
-    fprintf(output, "  printf(\"Application Options:\\n\");\n");
-    fprintf(output, "  printf(\"  -b, --benchmark     Run benchmark and show results\\n\");\n");
-    fprintf(output, "  printf(\"  -q, --quiet         Don't output anything except on failures\\n\");\n");
+    fprintf(output, "  fprintf(stderr, \"Usage:\\n\");\n");
+    fprintf(output, "  fprintf(stderr, \"  %%s [OPTION]\\n\", argv0);\n");
+    fprintf(output, "  fprintf(stderr, \"Help Options:\\n\");\n");
+    fprintf(output, "  fprintf(stderr, \"  -h, --help          Show help options\\n\");\n");
+    fprintf(output, "  fprintf(stderr, \"Application Options:\\n\");\n");
+    fprintf(output, "  fprintf(stderr, \"  -b, --benchmark     Run benchmark and show results\\n\");\n");
+    fprintf(output, "  fprintf(stderr, \"  -q, --quiet         Don't output anything except on failures\\n\");\n");
     fprintf(output, "\n");
     fprintf(output, "  exit(0);\n");
     fprintf(output, "}\n");
@@ -412,7 +434,7 @@ main (int argc, char *argv[])
     fprintf(output, "    }\n");
     fprintf(output, "  }\n");
     fprintf(output, "\n");
-    for(i=0;i<n;i++){
+    for(i=0;i<n_programs;i++){
       output_code_test (programs[i], output);
     }
     fprintf(output, "\n");
@@ -423,12 +445,12 @@ main (int argc, char *argv[])
     fprintf(output, "}\n");
   } else if (mode == MODE_ASSEMBLY) {
     fprintf(output, "%s", orc_target_get_asm_preamble (target));
-    for(i=0;i<n;i++){
+    for(i=0;i<n_programs;i++){
       output_code_assembly (programs[i], output);
     }
   }
 
-  for(i=0;i<n;i++){
+  for(i=0;i<n_programs;i++){
     orc_program_free(programs[i]);
   }
   free(programs);
@@ -772,7 +794,7 @@ output_code_backup (OrcProgram *p, FILE *output)
     if (ORC_COMPILE_RESULT_IS_SUCCESSFUL(result)) {
       fprintf(output, "%s\n", orc_program_get_asm_code (p));
     } else {
-      printf("Failed to compile backup code for '%s'\n", p->name);
+      fprintf(stderr, "Failed to compile backup code for '%s'\n", p->name);
       error = TRUE;
     }
   }
@@ -798,7 +820,7 @@ output_code_no_orc (OrcProgram *p, FILE *output)
     if (ORC_COMPILE_RESULT_IS_SUCCESSFUL(result)) {
       fprintf(output, "%s\n", orc_program_get_asm_code (p));
     } else {
-      printf("Failed to compile no orc for '%s'\n", p->name);
+      fprintf(stderr, "Failed to compile no orc for '%s'\n", p->name);
       error = TRUE;
     }
   }
@@ -862,33 +884,33 @@ output_code_execute (OrcProgram *p, FILE *output, int is_inline)
       fprintf(output, "  OrcProgram *p = _orc_program_%s;\n", p->name);
     }
   } else {
-    fprintf(output, "  static volatile int p_inited = 0;\n");
+    fprintf(output, "  static OrcOnce once = ORC_ONCE_INIT;\n");
     if (use_code) {
-      fprintf(output, "  static OrcCode *c = 0;\n");
+      fprintf(output, "  OrcCode *c;\n");
     } else {
-      fprintf(output, "  static OrcProgram *p = 0;\n");
+      fprintf(output, "  OrcProgram *p;\n");
     }
   }
   fprintf(output, "  void (*func) (OrcExecutor *);\n");
   fprintf(output, "\n");
   if (use_lazy_init) {
-    fprintf(output, "  if (!p_inited) {\n");
-    fprintf(output, "    orc_once_mutex_lock ();\n");
-    fprintf(output, "    if (!p_inited) {\n");
     if (use_code) {
-      fprintf(output, "      OrcProgram *p;\n");
+      fprintf(output, "  if (!orc_once_enter (&once, (void **) &c)) {\n");
+      fprintf(output, "    OrcProgram *p;\n");
+    } else {
+      fprintf(output, "  if (!orc_once_enter (&once, (void **) &p)) {\n");
     }
     fprintf(output, "\n");
     output_program_generation (p, output, is_inline);
     fprintf(output, "\n");
-    fprintf(output, "      orc_program_compile (p);\n");
+    fprintf(output, "    orc_program_compile (p);\n");
     if (use_code) {
-      fprintf(output, "      c = orc_program_take_code (p);\n");
-      fprintf(output, "      orc_program_free (p);\n");
+      fprintf(output, "    c = orc_program_take_code (p);\n");
+      fprintf(output, "    orc_program_free (p);\n");
+      fprintf(output, "    orc_once_leave (&once, c);\n");
+    } else {
+      fprintf(output, "    orc_once_leave (&once, p);\n");
     }
-    fprintf(output, "    }\n");
-    fprintf(output, "    p_inited = TRUE;\n");
-    fprintf(output, "    orc_once_mutex_unlock ();\n");
     fprintf(output, "  }\n");
   }
   if (use_code) {
@@ -1008,10 +1030,10 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
 
     fprintf(output, "#if 1\n");
     /* fprintf(output, "#ifdef bytecode\n"); */
-    fprintf(output, "      static const orc_uint8 bc[] = {\n");
+    fprintf(output, "    static const orc_uint8 bc[] = {\n");
     for(i=0;i<bytecode->length;i++) {
       if ((i&0xf) == 0) {
-        fprintf(output, "        ");
+        fprintf(output, "      ");
       }
       fprintf(output, "%d, ", bytecode->bytecode[i]);
       if ((i&0xf) == 15) {
@@ -1021,11 +1043,11 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
     if ((i&0xf) != 15) {
       fprintf(output, "\n");
     }
-    fprintf(output, "      };\n");
-    fprintf(output, "      p = orc_program_new_from_static_bytecode (bc);\n");
+    fprintf(output, "    };\n");
+    fprintf(output, "    p = orc_program_new_from_static_bytecode (bc);\n");
     /* fprintf(output, "     orc_program_set_name (p, \"%s\");\n", p->name); */
     if (use_backup && !is_inline) {
-      fprintf(output, "      orc_program_set_backup_function (p, _backup_%s);\n",
+      fprintf(output, "    orc_program_set_backup_function (p, _backup_%s);\n",
           p->name);
     }
 
@@ -1036,10 +1058,10 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
       OrcBytecode *bytecode2 = bytecode2 = orc_bytecode_from_program (p2);
 
       fprintf(output, "#ifdef badbytecode\n");
-      fprintf(output, "    static const orc_uint8 bc[] = {\n");
+      fprintf(output, "  static const orc_uint8 bc[] = {\n");
       for(i=0;i<bytecode2->length;i++) {
         if ((i&0xf) == 0) {
-          fprintf(output, "      ");
+          fprintf(output, "    ");
         }
         fprintf(output, "%s%d, ",
             (bytecode->bytecode[i] == bytecode2->bytecode[i]) ? "" : "/* */",
@@ -1051,7 +1073,7 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
       if ((i&0xf) != 15) {
         fprintf(output, "\n");
       }
-      fprintf(output, "    };\n");
+      fprintf(output, "  };\n");
       fprintf(output, "#endif\n");
     }
 #endif
@@ -1061,36 +1083,36 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
     fprintf(output, "#else\n");
   }
 
-  fprintf(output, "      p = orc_program_new ();\n");
+  fprintf(output, "    p = orc_program_new ();\n");
   if (p->constant_n != 0) {
-    fprintf(output, "      orc_program_set_constant_n (p, %d);\n",
+    fprintf(output, "    orc_program_set_constant_n (p, %d);\n",
         p->constant_n);
   }
   if (p->n_multiple != 0) {
     REQUIRE(0,4,14,1);
-    fprintf(output, "      orc_program_set_n_multiple (p, %d);\n",
+    fprintf(output, "    orc_program_set_n_multiple (p, %d);\n",
         p->n_multiple);
   }
   if (p->n_minimum != 0) {
     REQUIRE(0,4,14,1);
-    fprintf(output, "      orc_program_set_n_minimum (p, %d);\n",
+    fprintf(output, "    orc_program_set_n_minimum (p, %d);\n",
         p->n_minimum);
   }
   if (p->n_maximum != 0) {
     REQUIRE(0,4,14,1);
-    fprintf(output, "      orc_program_set_n_maximum (p, %d);\n",
+    fprintf(output, "    orc_program_set_n_maximum (p, %d);\n",
         p->n_maximum);
   }
   if (p->is_2d) {
-    fprintf(output, "      orc_program_set_2d (p);\n");
+    fprintf(output, "    orc_program_set_2d (p);\n");
     if (p->constant_m != 0) {
-      fprintf(output, "      orc_program_set_constant_m (p, %d);\n",
+      fprintf(output, "    orc_program_set_constant_m (p, %d);\n",
           p->constant_m);
     }
   }
-  fprintf(output, "      orc_program_set_name (p, \"%s\");\n", p->name);
+  fprintf(output, "    orc_program_set_name (p, \"%s\");\n", p->name);
   if (use_backup && !is_inline) {
-    fprintf(output, "      orc_program_set_backup_function (p, _backup_%s);\n",
+    fprintf(output, "    orc_program_set_backup_function (p, _backup_%s);\n",
         p->name);
   }
   for(i=0;i<4;i++){
@@ -1098,10 +1120,10 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
     if (var->size) {
       if (var->alignment != var->size) {
         REQUIRE(0,4,14,1);
-        fprintf(output, "      orc_program_add_destination_full (p, %d, \"%s\", 0, %d);\n",
+        fprintf(output, "    orc_program_add_destination_full (p, %d, \"%s\", 0, %d);\n",
             var->size, varnames[ORC_VAR_D1 + i], var->alignment);
       } else {
-        fprintf(output, "      orc_program_add_destination (p, %d, \"%s\");\n",
+        fprintf(output, "    orc_program_add_destination (p, %d, \"%s\");\n",
             var->size, varnames[ORC_VAR_D1 + i]);
       }
     }
@@ -1111,11 +1133,11 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
     if (var->size) {
       if (var->alignment != var->size) {
         REQUIRE(0,4,14,1);
-        fprintf(output, "      orc_program_add_source_full (p, %d, \"%s\", 0, %d);\n",
+        fprintf(output, "    orc_program_add_source_full (p, %d, \"%s\", 0, %d);\n",
             var->size, varnames[ORC_VAR_S1 + i],
             var->alignment);
       } else {
-        fprintf(output, "      orc_program_add_source (p, %d, \"%s\");\n",
+        fprintf(output, "    orc_program_add_source (p, %d, \"%s\");\n",
             var->size, varnames[ORC_VAR_S1 + i]);
       }
     }
@@ -1123,7 +1145,7 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
   for(i=0;i<4;i++){
     var = &p->vars[ORC_VAR_A1 + i];
     if (var->size) {
-      fprintf(output, "      orc_program_add_accumulator (p, %d, \"%s\");\n",
+      fprintf(output, "    orc_program_add_accumulator (p, %d, \"%s\");\n",
           var->size, varnames[ORC_VAR_A1 + i]);
     }
   }
@@ -1131,11 +1153,11 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
     var = &p->vars[ORC_VAR_C1 + i];
     if (var->size == 0) continue;
     if (var->size <= 4) {
-      fprintf(output, "      orc_program_add_constant (p, %d, 0x%08x, \"%s\");\n",
+      fprintf(output, "    orc_program_add_constant (p, %d, 0x%08x, \"%s\");\n",
           var->size, (int)var->value.i, varnames[ORC_VAR_C1 + i]);
     } else if (var->size > 4) {
       REQUIRE(0,4,8,1);
-      fprintf(output, "      orc_program_add_constant_int64 (p, %d, 0x%08x%08xULL, \"%s\");\n",
+      fprintf(output, "    orc_program_add_constant_int64 (p, %d, 0x%08x%08xULL, \"%s\");\n",
           var->size, (orc_uint32)(((orc_uint64)var->value.i)>>32),
           (orc_uint32)(var->value.i), varnames[ORC_VAR_C1 + i]);
     }
@@ -1163,14 +1185,14 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
         default:
           ORC_ASSERT(0);
       }
-      fprintf(output, "      orc_program_add_parameter%s (p, %d, \"%s\");\n",
+      fprintf(output, "    orc_program_add_parameter%s (p, %d, \"%s\");\n",
           suffix, var->size, varnames[ORC_VAR_P1 + i]);
     }
   }
   for(i=0;i<16;i++){
     var = &p->vars[ORC_VAR_T1 + i];
     if (var->size) {
-      fprintf(output, "      orc_program_add_temporary (p, %d, \"%s\");\n",
+      fprintf(output, "    orc_program_add_temporary (p, %d, \"%s\");\n",
           var->size, varnames[ORC_VAR_T1 + i]);
     }
   }
@@ -1185,11 +1207,11 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
       }
 
       if (p->vars[insn->src_args[1]].size != 0) {
-        fprintf(output, "      orc_program_append (p, \"%s\", %s, %s, %s);\n",
+        fprintf(output, "    orc_program_append (p, \"%s\", %s, %s, %s);\n",
             insn->opcode->name, enumnames[insn->dest_args[0]],
             enumnames[insn->src_args[0]], enumnames[insn->src_args[1]]);
       } else {
-        fprintf(output, "      orc_program_append_ds (p, \"%s\", %s, %s);\n",
+        fprintf(output, "    orc_program_append_ds (p, \"%s\", %s, %s);\n",
             insn->opcode->name, enumnames[insn->dest_args[0]],
             enumnames[insn->src_args[0]]);
       }
@@ -1213,7 +1235,7 @@ output_program_generation (OrcProgram *p, FILE *output, int is_inline)
         args[n_args++] = insn->src_args[2];
       }
 
-      fprintf(output, "      orc_program_append_2 (p, \"%s\", %d, %s, %s, %s, %s);\n",
+      fprintf(output, "    orc_program_append_2 (p, \"%s\", %d, %s, %s, %s, %s);\n",
           insn->opcode->name, insn->flags, enumnames[args[0]],
           enumnames[args[1]], enumnames[args[2]],
           enumnames[args[3]]);
@@ -1464,7 +1486,7 @@ output_code_assembly (OrcProgram *p, FILE *output)
     if (ORC_COMPILE_RESULT_IS_SUCCESSFUL(result)) {
       fprintf(output, "%s\n", orc_program_get_asm_code (p));
     } else {
-      printf("Failed to compile assembly for '%s'\n", p->name);
+      fprintf(stderr, "Failed to compile assembly for '%s'\n", p->name);
       error = TRUE;
     }
   }
